// Performance testing client process	
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <corbadii.h>



#ifdef SOLARIS
extern "C" int gettimeofday(struct timeval *);
#define GETTIME(x)	gettimeofday(x)
#else
extern "C" int gettimeofday(struct timeval *, struct timezone *);
#define GETTIME(x)	gettimeofday(x, (struct timezone *)NULL)
#endif


#define SIZE_STRUCT  (sizeof(short) + sizeof(long) + sizeof(float) + \
			sizeof(double) + sizeof(char) + strlen("String"))

#define NUM_ELEMENTS	100

struct timeval starttime;
struct timeval endtime;

int	num_samples = 100;
int	seq_size = 100;

double	total_time = 0.0;

void INIT_TIME()
{
	total_time = 0;
}

void START_TIME()
{
	GETTIME(&starttime);
}

void ADD_TIME()
{
	GETTIME(&endtime);
	total_time += ((endtime.tv_sec - starttime.tv_sec) * 1000.0) +
                        ((endtime.tv_usec - starttime.tv_usec)/ 1000.0);
}

void print_stats(const char *func_name, int num_bytes_per_sample) 
{

	cout << "\n\n----------- " << func_name << " ----------------" << endl;

	cout << "Average time in millisecs: " << (total_time / num_samples)
						<< endl; 
	if ( num_bytes_per_sample <= 0)
		return;
	double total_num_bytes = num_bytes_per_sample * num_samples;
	cout << "Total num of bytes sent: " << total_num_bytes << endl;
	cout << "Average num of bytes per sec: " 
		<< (long)(total_num_bytes * 1000.0 / total_time) << endl;
}


CORBA_DII::Request *make_req(const char *interface, const char *method)
{
	CORBA::Environment env;
	CORBA_DII::Request *req = CORBA_DII::Request::create(interface, method,
					"testrep", env);
	if ( env.check_exception() ) {
		cout << "Error creating request for " << interface << "::"
			<< method << endl;
		cout << env;
		exit(1);
	}
	req->bind();
	CORBA::Environment& reqEnv = req->_environment();
	if ( reqEnv.check_exception() ) {
		cout << "Error binding to " << interface << "::" 
			<< method << endl;
		req->_release();
		exit(1);
	}
	return req;
}

void oneway_no_param()
{
	CORBA_DII::Request *req = make_req("TestOneway", "test_no_param");

	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++)
		req->send_oneway();
	ADD_TIME();
	print_stats("Oneway::test_no_param", 0);
	req->_release();
}

void oneway_prim_args()
{
	CORBA_DII::Request *req = make_req("TestOneway", "test_prim_args");
	
	CORBA::ValueShort shortVal;
	CORBA::ValueLong longVal;
	CORBA::ValueFloat floatVal;
	CORBA::ValueDouble doubleVal;
	CORBA::ValueChar charVal;
	CORBA::ValueString stringVal("String");

	req->arg((CORBA::Long)0, shortVal);
	req->arg(1, longVal);
	req->arg(2, floatVal);
	req->arg(3, doubleVal);
	req->arg(4, charVal);
	req->arg(5, stringVal);

	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++)
		req->send_oneway();
	ADD_TIME();

	int num_bytes = sizeof(short) + sizeof(long) +
			sizeof(float) + sizeof(double) +
			sizeof(char) + strlen("String"); 

	print_stats("Oneway::test_prim_args", num_bytes);
	req->_release();
}

void oneway_struct()
{
	CORBA_DII::Request *req = make_req("TestOneway", "test_struct");

	CORBA::Value *structValue = req->arg((CORBA::Long)0);
	structValue->member("stringVal") =  "String";
	
	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++)
		req->send_oneway();
	ADD_TIME();

	print_stats("Oneway::test_struct", SIZE_STRUCT);
	req->_release();
}
	
void oneway_prim_seq()
{
	CORBA_DII::Request *req = make_req("TestOneway", "test_prim_seq");

	CORBA::Value *shortSeq = req->arg((CORBA::Long)0);
	CORBA::Value *longSeq = req->arg(1);
	CORBA::Value *floatSeq = req->arg(2);
	CORBA::Value *doubleSeq = req->arg(3);
	CORBA::Value *charSeq = req->arg(4);
	CORBA::Value *stringSeq = req->arg(5);

	for (int i=0; i< seq_size; i++) {
		shortSeq->add_member(new CORBA::ValueShort);
		longSeq->add_member(new CORBA::ValueLong);
		floatSeq->add_member(new CORBA::ValueFloat);
		doubleSeq->add_member(new CORBA::ValueDouble);
		charSeq->add_member(new CORBA::ValueChar);
		stringSeq->add_member(new CORBA::ValueString("String"));
	}


	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		req->send_oneway();
	ADD_TIME();
	

	int num_bytes = (sizeof(short) + sizeof(long) +
			sizeof(float) + sizeof(double) +
			sizeof(char) + strlen("String")) * seq_size;


	print_stats("Oneway::test_prim_seq", num_bytes);

	req->_release();
}

	
void oneway_struct_seq()
{
	CORBA_DII::Request *req = make_req("TestOneway", "test_struct_seq");

	CORBA::Value *structSeq = req->arg((CORBA::Long)0);
	const CORBA::SequenceTypeCode *seqTypeCode = 
		CORBA::SequenceTypeCode::_narrow(&structSeq->typeCode());
	const CORBA::TypeCode& structTypeCode = seqTypeCode->member_typecode();

	for (int i=0; i< NUM_ELEMENTS; i++) {
		CORBA::Value *structVal = CORBA::Value::factory(structTypeCode);
		structVal->member("stringVal") =  "String";
		structSeq->add_member(structVal);
	}

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		req->send_oneway();
	ADD_TIME();
	
	int num_bytes = SIZE_STRUCT * seq_size;
	print_stats("Oneway::test_struct_seq", num_bytes);
	req->_release();
}
	
void oneway_struct_array()
{
	CORBA_DII::Request *req = make_req("TestOneway", "test_struct_array");

	CORBA::Value *structArray = req->arg((CORBA::Long)0);
	for (int i=0; i< NUM_ELEMENTS; i++) 
		(*structArray)[i].member("stringVal") = "String";

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		req->send_oneway();
	ADD_TIME();

	
	int num_bytes = SIZE_STRUCT  * NUM_ELEMENTS;
	print_stats("Oneway::test_struct_array", num_bytes);
	req->_release();
}


void reqreply_prim_args()
{
	CORBA_DII::Request *req = make_req("TestReqReply", "test_prim_args");

	*req->arg("stringVal") = "String";
	*req->arg("inoutString") = "String";

	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++)  {
		req->invoke();
	}
	ADD_TIME();
	int num_bytes = 4 * (sizeof(short) + sizeof(long) + sizeof(float) +
				sizeof(double) + sizeof(char) 
				+ strlen("String"));
	print_stats("TestReqReply::test_prim_args", num_bytes);
	req->_release();
}

void reqreply_struct_args()
{
	CORBA_DII::Request *req = make_req("TestReqReply", "test_struct_args");

	req->arg((CORBA::Long)0)->member("stringVal") = "String";
	req->arg(1)->member("stringVal") = "String";

	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++) {
		req->invoke();
	}
	ADD_TIME();

	int num_bytes = 4 * SIZE_STRUCT;
	print_stats("TestReqReply::test_struct_args", num_bytes);
	req->_release();
}

void reqreply_prim_seq()
{
	CORBA_DII::Request *req = make_req("TestReqReply", "test_prim_seq");

	CORBA::Value	*in_shortSeq = req->arg((CORBA::Long)0);
	CORBA::Value	*in_longSeq = req->arg(1);
	CORBA::Value	*in_floatSeq = req->arg(2);
	CORBA::Value	*in_doubleSeq = req->arg(3);
	CORBA::Value	*in_charSeq = req->arg(4);
	CORBA::Value	*in_stringSeq = req->arg(5);

	CORBA::Value	*inout_shortSeq = req->arg(6);
	CORBA::Value	*inout_longSeq = req->arg(7);
	CORBA::Value	*inout_floatSeq = req->arg(8);
	CORBA::Value	*inout_doubleSeq = req->arg(9);
	CORBA::Value	*inout_charSeq = req->arg(10);
	CORBA::Value	*inout_stringSeq = req->arg(11);

	
	for (int i=0; i < seq_size; i++) {
		in_shortSeq->add_member(new CORBA::ValueShort);
		in_longSeq->add_member(new CORBA::ValueLong);
		in_floatSeq->add_member(new CORBA::ValueFloat);
		in_doubleSeq->add_member(new CORBA::ValueDouble);
		in_charSeq->add_member(new CORBA::ValueChar);
		in_stringSeq->add_member(new CORBA::ValueString("String"));
		
		inout_shortSeq->add_member(new CORBA::ValueShort);
		inout_longSeq->add_member(new CORBA::ValueLong);
		inout_floatSeq->add_member(new CORBA::ValueFloat);
		inout_doubleSeq->add_member(new CORBA::ValueDouble);
		inout_charSeq->add_member(new CORBA::ValueChar);
		inout_stringSeq->add_member(new CORBA::ValueString("String"));
	}

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++) {
		req->invoke();
	}
	ADD_TIME();

        // Num of bytes sent/received
        int num_bytes = 4 * seq_size * (sizeof(short) + sizeof(long) +
                        sizeof(float) + sizeof(double) + sizeof(char) +
                        strlen("String"));


	print_stats("TestReqReply::test_prim_seq", num_bytes);
	req->_release();
}

void reqreply_struct_seq()
{
	CORBA_DII::Request *req = make_req("TestReqReply", "test_struct_seq");

	CORBA::Value *structSeq = req->arg((CORBA::Long)0);
	const CORBA::SequenceTypeCode *seqTypeCode = 
		CORBA::SequenceTypeCode::_narrow(&structSeq->typeCode());
	const CORBA::TypeCode& structTypeCode = seqTypeCode->member_typecode();

	CORBA::Value *inout_structSeq = req->arg(1);
	const CORBA::SequenceTypeCode *inout_seqTypeCode = 
		CORBA::SequenceTypeCode::_narrow(&inout_structSeq->typeCode());
	const CORBA::TypeCode& inout_structTypeCode = 
		inout_seqTypeCode->member_typecode();

	for (int i=0; i< seq_size; i++) {
		CORBA::Value *in = CORBA::Value::factory(structTypeCode);
		in->member("stringVal") = "String";
		CORBA::Value *inout = CORBA::Value::factory(structTypeCode);
		inout->member("stringVal") = "String";
		structSeq->add_member(in);
		inout_structSeq->add_member(inout);
	}

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++) {
		req->invoke();
	}
	ADD_TIME();

	int num_bytes = 4 * SIZE_STRUCT * seq_size;
	
	print_stats("TestReqReply::test_struct_seq", num_bytes);
	req->_release();
}

void reqreply_struct_array()
{
	CORBA_DII::Request *req = make_req("TestReqReply", "test_struct_array");

	CORBA::Value *structArray = req->arg((CORBA::Long)0);
	for (int i=0; i< NUM_ELEMENTS; i++) 
		(*structArray)[i].member("stringVal") = "String";


	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++) {
		req->invoke();
	}
	ADD_TIME();

	int num_bytes = 4 * SIZE_STRUCT * NUM_ELEMENTS;
	
	print_stats("TestReqReply::test_struct_array", num_bytes);
	req->_release();
}

void sighandler(int)
{
	exit(0);
}

	
int main(int argc, char **argv)
{
	signal(SIGINT, sighandler);
	int c;
	while ((c = getopt(argc, argv, "n:s:?")) != EOF) {
		switch (c) {
			case 'n':
				num_samples = atoi(optarg);
				break;
			case 's':
				seq_size = atoi(optarg);
				break;
			case '?':
			default:
				cout << "Usage: perfclnt [-n num_samples] "
					<< "[-s seq_size]" << endl;
				return 0;
		}
	}
	
	oneway_no_param();
	oneway_prim_args();
	oneway_struct();
	oneway_prim_seq();
	oneway_struct_seq();
	oneway_struct_array();


	reqreply_prim_args();
	reqreply_struct_args();
	reqreply_prim_seq();
	reqreply_struct_seq();
	reqreply_struct_array();

	return 0;
}

